Package: abseqR
Type: Package
Title: Reporting and data analysis functionalities for Rep-Seq datasets
        of antibody libraries
Version: 1.30.0
Authors@R: c(
    person("JiaHong", "Fong", email = "jiahfong@gmail.com", role = c("cre", "aut")),
    person("Monther", "Alhamdoosh", email = "m.hamdoosh@gmail.com", role = "aut"))
Description: AbSeq is a comprehensive bioinformatic pipeline for the analysis of sequencing datasets 
    generated from antibody libraries and abseqR is one of its packages. abseqR empowers the users 
    of abseqPy (https://github.com/malhamdoosh/abseqPy) with plotting and reporting capabilities and
    allows them to generate interactive HTML reports for the convenience of viewing and sharing with 
    other researchers. Additionally, abseqR extends abseqPy to compare multiple
    repertoire analyses and perform further downstream analysis on its output.
License: GPL-3 | file LICENSE
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: ggplot2, RColorBrewer, circlize, reshape2, VennDiagram, plyr,
        flexdashboard, BiocParallel (>= 1.1.25), png, grid, gridExtra,
        rmarkdown, knitr, vegan, ggcorrplot, ggdendro, plotly,
        BiocStyle, stringr, utils, methods, grDevices, stats, tools,
        graphics
VignetteBuilder: knitr
RoxygenNote: 6.1.0
Collate: 'accessors-AbSeq.R' 'AbSeqCRep.R' 'util.R' 'distributions.R'
        'upstreamAnalysis.R' 'productivityAnalysis.R'
        'primerAnalysis.R' 'diversityAnalysis.R' 'annotationAnalysis.R'
        'abundanceAnalysis.R' 'plotter.R' 'AbSeqRep.R' 'abseqReport.R'
        'statistics.R' 'pairwise.R'
SystemRequirements: pandoc (>= 1.19.2.1)
URL: https://github.com/malhamdoosh/abseqR
BugReports: https://github.com/malhamdoosh/abseqR/issues
biocViews: Sequencing, Visualization, ReportWriting, QualityControl,
        MultipleComparison
Suggests: testthat
git_url: https://git.bioconductor.org/packages/abseqR
git_branch: RELEASE_3_23
git_last_commit: b90f5f9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:33:23 UTC; biocbuild
Author: JiaHong Fong [cre, aut],
  Monther Alhamdoosh [aut]
Maintainer: JiaHong Fong <jiahfong@gmail.com>
