% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{.emptyPlot}
\alias{.emptyPlot}
\title{Creates and returns an empty plot}
\usage{
.emptyPlot()
}
\value{
empty ggplot2 object
}
\description{
Creates and returns an empty plot
}
